CCTimerOptionsEffect = class(Turbine.UI.Control);

local function fnFocusGained(self, args)
	self:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.value = self:GetText();
end

local function fnFocusLost(self, args)
	self:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	self:SetText(self.value);
	self.value = nil;
end

local function fnKeyDown(self, args)
	if args.Action == 162 then
		local x = self.parent.outgoing(self:GetText());

		if x ~= false then
			self.parent.parent:SetEffect(self.parent.icon, self.parent.name, self.label, x);
			self.value = self.parent.incoming(x);
		end

		self.parent.parent:Focus();
	end
end

function CCTimerOptionsEffect:Constructor(parent, w, h, icon, name, labels, values, outgoing, incoming)
	Turbine.UI.Control.Constructor(self);

	self.parent = parent;
	self.h = h;
	self.w = w;

	self.icon = icon;
	self.name = name;
	self.outgoing = outgoing;
	self.incoming = incoming;

	self.iconControl = Turbine.UI.Control();
	self.iconControl:SetPosition(5, 0);
	self.iconControl:SetSize(h, h);
	self.iconControl:SetBackground(icon);
	self.iconControl:SetParent(self);

	self.nameField = Turbine.UI.Lotro.TextBox();
	self.nameField:SetPosition(h + 5, 0);
	self.nameField:SetHeight(h);
	self.nameField:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.nameField:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	self.nameField:SetReadOnly(true);
	self.nameField:SetText(name);
	self.nameField:SetParent(self);

	self.fields = { };
	for i, label in ipairs(labels) do
		local field = Turbine.UI.Lotro.TextBox();
		field:SetSize(w, h);
		field:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
		field:SetFont(Turbine.UI.Lotro.Font.Verdana16);
		field.label = label;
		field.parent = self;
		field.value = incoming(values[i]);
		field:SetText(field.value);
		field.FocusGained = fnFocusGained;
		field.FocusLost = fnFocusLost;
		field.KeyDown = fnKeyDown;
		field:SetParent(self);
		table.insert(self.fields, field);
	end

	self:SetHeight(h);
	self:SetParent(parent);
end

function CCTimerOptionsEffect:SizeChanged()
	local w = self:GetWidth();
	local w1 = math.max(self.w, w - (self.h + #self.fields * (self.w + 5) + 10));

	self.nameField:SetWidth(w1);

	local x = self.h + w1 + 5 + 5;
	for _, field in ipairs(self.fields) do
		field:SetPosition(x, 0);
		x = x + self.w + 5;
	end
end
