CCTimerRegime = class();

function CCTimerRegime:Constructor(parent)
	self.parent = parent;

	self.brightness = 1.0;

	self.top = nil;
	self.bottom = nil;

	self.visible = false;

	self.shown = false;
end

function CCTimerRegime:SetColor(color)
	self.color = color;

	self.parent:ColorUpdate();
end

function CCTimerRegime:SetBrightness(brightness)
	self.brightness = brightness;

	self.parent:ColorUpdate();
end

function CCTimerRegime:SetBounds(top, bottom)
	if self.top ~= top or self.bottom ~= bottom then
		self.top = top;
		self.bottom = bottom;

		self.parent:IntersectionUpdate();
	end
end

function CCTimerRegime:GetBounds()
	return self.top, self.bottom;
end

function CCTimerRegime:SetVisible(visible)
	if self.visible ~= visible then
		self.visible = visible;

		self.parent:IntersectionUpdate();
	end
end

function CCTimerRegime:IsVisible()
	return self.visible;
end

CCTimerRegimeIntersection = class(Turbine.UI.Control);

function CCTimerRegimeIntersection:Constructor(parent, regimes)
	Turbine.UI.Control.Constructor(self);

	self.parent = parent;
	self.regimes = regimes;
	self.brightness = 1.0;

	self:SetParent(parent);
	self:SetWidth(parent:GetWidth());
	self:SetLeft(0);
	self.n = { };
	self:ColorFinal();
	self:SetVisible(false);
	self:SetMouseVisible(parent:IsMouseVisible());
end

function CCTimerRegimeIntersection:Destructor()
	self:SetParent(nil);
end

function CCTimerRegimeIntersection:SetBrightness(brightness)
	self.brightness = brightness;
	self:ColorFinal();
end

function CCTimerRegimeIntersection:GetBounds()
	if #self.regimes == 1 then
		local t, b = self.regimes[1]:GetBounds();

		if t == nil or b == nil then
			return t, b;
		end
	end

	local t = self:GetTop();
	local h = self:GetHeight();
	return t, (t + h - 1);
end

local function distance(x, y, m)
	local d = math.abs(x - y);
	if d > (m / 2) then
		d = m - d;
	end
	return d;
end

function CCTimerRegimeIntersection:ColorInit()
	local m = #self.parent.colors;

	self.n = { };
	self.curBrightness = 1.0;
	self.score = { };
	if self.parent.color ~= nil then
		for i = 1, m, 1 do
			if i ~= self.parent.color then
				self.score[i] = m - distance(i, self.parent.color, m);
			else
				self.score[i] = math.huge;
			end
		end
	else
		for i = 1, m, 1 do
			self.score[i] = 0;
		end
	end

	for _, regime in ipairs(self.regimes) do
		if regime.color ~= nil then
			table.insert(self.n, tostring(regime));
			self.color = regime.color;
			for i = 1, m, 1 do
				local d = distance(i, regime.color, m);
				self.score[i] = self.score[i] + ((d * d) * m) * self:GetHeight();
			end
		end
		self.curBrightness = self.curBrightness * regime.brightness;
	end

	if #self.n <= 1 then
		self.score = nil;
		if #self.n == 0 then
			self.color = self.parent.color;
		end
	else
		self.color = nil;
		table.sort(self.n);
	end
end

function CCTimerRegimeIntersection:ColorFinal()
	if self.color == nil or #self.n == 0 then
		self:SetVisible(false);
	else
		local b = self.curBrightness * self.brightness * self.parent.brightness;
		local color = self.parent.colors[self.color];
		self:SetBackColor(Turbine.UI.Color(self.parent.alpha, color.R * b, color.G * b, color.B * b));
		self:SetVisible(true);
	end
end

function CCTimerRegimeIntersection:MouseClick(args)
	if self.parent.MouseClick ~= nil then
		self.parent:MouseClick(args, self);
	end
end

function CCTimerRegimeIntersection:MouseEnter(args)
	if self.parent.MouseEnter ~= nil then
		self.parent:MouseEnter(args, self);
	end
end

function CCTimerRegimeIntersection:MouseLeave(args)
	if self.parent.MouseLeave ~= nil then
		self.parent:MouseLeave(args, self);
	end
end

CCTimerRegimeContainer = class(Turbine.UI.Control);

function CCTimerRegimeContainer:Constructor(parent, colors)
	Turbine.UI.Control.Constructor(self);

	self.regimes = { };
	self.intersections = { };

	self.alpha = 1.0;
	self.brightness = 1.0;

	self.n = { };

	self.colors = colors;

	self:SetParent(parent);
end

function CCTimerRegimeContainer:SetWidth(w)
	Turbine.UI.Control.SetWidth(self, w);

	for _, intersection in ipairs(self.intersections) do
		intersection:SetWidth(w);
	end
end

function CCTimerRegimeContainer:SetHeight(h)
	Turbine.UI.Control.SetHeight(self, h);

	self:IntersectionUpdate();
end

function CCTimerRegimeContainer:SetSize(w, h)
	self:SetWidth(w);
	self:SetHeight(h);
end

function CCTimerRegimeContainer:SetMouseVisible(v)
	Turbine.UI.Control.SetMouseVisible(self, v);
	for _, intersection in ipairs(self.intersections) do
		intersection:SetMouseVisible(v);
	end
end

function CCTimerRegimeContainer:SetColor(color)
	self.color = color;
	self:ColorUpdate();
end

function CCTimerRegimeContainer:SetAlpha(alpha)
	self.alpha = alpha;
	self:ColorUpdate();
end

function CCTimerRegimeContainer:SetBrightness(brightness)
	self.brightness = brightness;
	self:ColorUpdate();
end

local function getSkips(colors, m)
	local skip = { };

	for _, color in pairs(colors) do
		skip[color - 1] = true;
		skip[color + 0] = true;
		skip[color + 1] = true;
	end

	if skip[0] then
		skip[m] = true;
	end
	if skip[m + 1] then
		skip[1] = true;
	end

	return skip;
end

function CCTimerRegimeContainer:ColorUpdate()
	local m = #self.colors;

	if self.color ~= nil then
		local color = self.colors[self.color];
		self:SetBackColor(Turbine.UI.Color(self.alpha, color.R * self.brightness, color.G * self.brightness, color.B * self.brightness));
	end

	for _, intersection in ipairs(self.intersections) do
		intersection:ColorInit();
	end

	local groupedIntersections = { };
	for _, intersection in ipairs(self.intersections) do
		if intersection.score ~= nil then
			local group = { };
			local id = table.concat(intersection.n, ';');

			group.intersections = { };

			group.score = intersection.score;
			group.prev = intersection.prev;
			group.top = intersection.top;
			group.next = intersection.next;
			group.bottom = intersection.bottom;
			table.insert(group.intersections, intersection);

			intersection.score = nil;

			while intersection.next.score ~= nil and table.concat(intersection.next.n, ';') == id do
				intersection = intersection.next;

				group.next = intersection.next;
				group.bottom = intersection.bottom;
				for i, score in ipairs(intersection.score) do
					group.score[i] = group.score[i] + (score - (score % m));
				end
				table.insert(group.intersections, intersection);

				intersection.score = nil;
			end

			if #groupedIntersections > 0 and (groupedIntersections[#groupedIntersections].bottom + 1) == group.top then
				group.prev = groupedIntersections[#groupedIntersections];
				groupedIntersections[#groupedIntersections].next = group;
				table.insert(groupedIntersections, group);
			elseif group.prev.score ~= nil or group.next.score ~= nil then
				table.insert(groupedIntersections, group);
			else
				local skip = getSkips({ intersection.prev.color, intersection.next.color }, m);

				local scoreBest, colorBest;
				for color, score in ipairs(group.score) do
					if not skip[color] then
						if scoreBest == nil or score < scoreBest then
							scoreBest = score;
							colorBest = color;
						end
					end
				end

				for _, intersection in ipairs(group.intersections) do
					intersection.color = colorBest;
				end
			end
		end
	end

	repeat
		local change = false;

		for _, group in ipairs(groupedIntersections) do
			if group.next.score ~= nil then
				local groupA = group;
				local groupB = group.next;
				local scorePrev;

				if groupA.color ~= nil and groupB.color ~= nil then
					scorePrev = groupA.score[groupA.color] + groupB.score[groupB.color];
				end

				local skipA = getSkips({ groupA.prev.color }, m);
				local scoreBest;
				local colorABest;
				local colorBBest;
				for colorA, scoreA in ipairs(groupA.score) do
					if not skipA[colorA] then
						local skipB = getSkips({ colorA, groupB.next.color }, m);
						for colorB, scoreB in ipairs(groupB.score) do
							if not skipB[colorB] then
								if scoreBest == nil or (scoreA + scoreB) < scoreBest then
									scoreBest = scoreA + scoreB;
									colorABest = colorA;
									colorBBest = colorB;
								end
							end
						end
					end
				end

				if scorePrev == nil or scoreBest < scorePrev then
					groupA.color = colorABest;
					groupB.color = colorBBest;
					change = true;
				end
			end
		end
	until not change;

	for _, group in ipairs(groupedIntersections) do
		for _, intersection in ipairs(group.intersections) do
			intersection.color = group.color;
		end
	end

	for _, intersection in ipairs(self.intersections) do
		intersection:ColorFinal();
	end
end

function CCTimerRegimeContainer:GetColor(x)
	for _, intersection in ipairs(self.intersections) do
		if intersection.top <= x and (intersection.bottom + 1) > x then
			if intersection.color ~= nil then
				return self.colors[intersection.color];
			else
				return self.colors[self.color];
			end
		end
	end
	return self.colors[self.color];
end

function CCTimerRegimeContainer:AddRegime()
	local r = CCTimerRegime(self);

	table.insert(self.regimes, r);
	return r;
end

function CCTimerRegimeContainer:DeleteRegime(r)
	for i, regime in ipairs(self.regimes) do
		if regime == r then
			table.remove(self.regimes, i);
			if regime.visible and (regime.top ~= nil or regime.bottom ~= nil) then
				self:IntersectionUpdate();
			end
			return;
		end
	end
end

function CCTimerRegimeContainer:IntersectionUpdate()
	for _, intersection in ipairs(self.intersections) do
		intersection:Destructor();
	end
	self.intersections = { };

	local t = { };
	for i, regime in ipairs(self.regimes) do
		if regime.visible and (regime.top ~= nil or regime.bottom ~= nil) then
			if regime.top == nil then
				local b = 0;
				for _, r in ipairs(self.regimes) do
					if r.visible and r.bottom ~= nil and r.bottom < regime.bottom and r.bottom >= b then
						b = r.bottom + 1;
					end
				end
				table.insert(t, { x = b, add = true, regime = i });
			else
				table.insert(t, { x = regime.top, add = true, regime = i });
			end

			if regime.bottom == nil then
				local b = self:GetHeight() - 1;
				for _, r in ipairs(self.regimes) do
					if r.visible and r.top ~= nil and r.top > regime.top and r.top < b then
						b = r.top;
					end
				end
				table.insert(t, { x = b, add = false, regime = i });
			else
				table.insert(t, { x = regime.bottom + 1, add = false, regime = i });
			end
		end
	end
	table.sort(t, function(a, b) return a.x > b.x; end);

	local regimes = { };
	while #t > 0 do
		local x = t[#t].x;
		while #t > 0 and t[#t].x == x do
			local i = table.remove(t);
			if i.add then
				regimes[i.regime] = true;
			else
				regimes[i.regime] = nil;
			end
		end

		if #t > 0 then
			local r = { };
			for i in pairs(regimes) do
				table.insert(r, self.regimes[i]);
			end
			if #r > 0 then
				local i = CCTimerRegimeIntersection(self, r);
				i.top = x;
				i.bottom = t[#t].x - 1;
				i:SetTop(i.top);
				i:SetHeight((i.bottom - i.top) + 1);
				table.insert(self.intersections, i);
			end
		end
	end

	for i, intersection in ipairs(self.intersections) do
		if i ~= 1 and self.intersections[i - 1].bottom == (intersection.top - 1) then
			intersection.prev = self.intersections[i - 1];
		else
			intersection.prev = self;
		end

		if i ~= #self.intersections and self.intersections[i + 1].top == (intersection.bottom + 1) then
			intersection.next = self.intersections[i + 1];
		else
			intersection.next = self;
		end
	end

	self:ColorUpdate();
end